@@ -161,9 +161,60 @@ on_client_registered (GObject             *source_object,
                                           manager);
 
                 g_free (object_path);
+                g_variant_unref (variant);
         }
 }
 
+static void
+session_env_done (GObject             *source_object,
+                  GAsyncResult        *res,
+                  gpointer             user_data)
+{
+        GVariant *result;
+        GError *error = NULL;
+
+        result = g_dbus_proxy_call_finish (G_DBUS_PROXY (source_object), res, &error);
+        if (error != NULL)
+                g_error_free (error);
+
+        g_variant_unref (result);
+}
+
+static void
+set_session_env (GDBusProxy  *proxy,
+                 const gchar *name,
+                 const gchar *value)
+{
+        g_dbus_proxy_call (proxy,
+                           "Setenv",
+                           g_variant_new ("(ss)", name, value),
+                           G_DBUS_CALL_FLAGS_NONE,
+                           -1,
+                           NULL,
+                           (GAsyncReadyCallback) session_env_done,
+                           NULL);
+}
+
+static void
+set_locale (GDBusProxy *proxy)
+{
+        GSettings *locale_settings;
+        gchar *region;
+
+        /* Set locale environment */
+        locale_settings = g_settings_new ("org.gnome.system.locale");
+        region = g_settings_get_string (locale_settings, "region");
+        if (region && region[0]) {
+                /* Only set the locale settings if the user has ever customized them */
+                set_session_env (proxy, "LC_TIME", region);
+                set_session_env (proxy, "LC_NUMERIC", region);
+                set_session_env (proxy, "LC_MONETARY", region);
+                set_session_env (proxy, "LC_MEASUREMENT", region);
+        }
+
+        g_object_unref (locale_settings);
+}
+
 static void
 got_session_proxy (GObject      *source_object,
                    GAsyncResult *res,
@@ -179,6 +230,11 @@ got_session_proxy (GObject      *source_object,
         } else {
                 const char *startup_id;
 
+                /* Always call this first, as Setenv can only be called before
+                   any client registers */
+                set_locale (proxy);
+
+                /* Register the daemon with gnome-session */
                 g_signal_connect (G_OBJECT (proxy), "g-signal",
                                   G_CALLBACK (on_session_over), NULL);
                 startup_id = g_getenv ("DESKTOP_AUTOSTART_ID");
@@ -198,10 +254,6 @@ on_term_signal_pipe_closed (GIOChannel *source,
                             GIOCondition condition,
                             gpointer data)
 {
-        GnomeSettingsManager *manager;
-
-        manager = GNOME_SETTINGS_MANAGER (data);
-
         term_signal_pipe_fds[0] = -1;
 
         g_debug ("Received SIGTERM - shutting down");
