@@ -535,6 +535,7 @@ ax_slowkeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
         manager->priv->notification = notify_notification_new (title,
                                                                message,
                                                                "preferences-desktop-accessibility");
+        notify_notification_set_app_name (manager->priv->notification, _("Universal Access"));
         notify_notification_set_timeout (manager->priv->notification, NOTIFICATION_TIMEOUT * 1000);
 
         notify_notification_add_action (manager->priv->notification,
@@ -1022,6 +1023,11 @@ gsd_a11y_keyboard_manager_stop (GsdA11yKeyboardManager *manager)
 
         g_debug ("Stopping a11y_keyboard manager");
 
+        if (p->start_idle_id != 0) {
+                g_source_remove (p->start_idle_id);
+                p->start_idle_id = 0;
+        }
+
         if (p->device_manager != NULL) {
                 g_signal_handler_disconnect (p->device_manager, p->device_added_id);
                 p->device_manager = NULL;
@@ -1033,6 +1039,7 @@ gsd_a11y_keyboard_manager_stop (GsdA11yKeyboardManager *manager)
         }
 
         if (p->settings != NULL) {
+                g_signal_handlers_disconnect_by_func (p->settings, keyboard_callback, manager);
                 g_object_unref (p->settings);
                 p->settings = NULL;
         }
