@@ -241,6 +241,7 @@ require_restart (GsdUpdatesFirmware *firmware)
 
         /* TRANSLATORS: title of libnotify bubble */
         notification = notify_notification_new (_("Additional software was installed"), message, NULL);
+        notify_notification_set_app_name (notification, _("Software Updates"));
         notify_notification_set_timeout (notification, NOTIFY_EXPIRES_NEVER);
         notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
 
@@ -265,6 +266,7 @@ require_replug (GsdUpdatesFirmware *firmware)
 
         /* TRANSLATORS: title of libnotify bubble */
         notification = notify_notification_new (_("Additional software was installed"), message, NULL);
+        notify_notification_set_app_name (notification, _("Software Updates"));
         notify_notification_set_timeout (notification, NOTIFY_EXPIRES_NEVER);
         notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
 
@@ -289,6 +291,7 @@ require_nothing (GsdUpdatesFirmware *firmware)
 
         /* TRANSLATORS: title of libnotify bubble */
         notification = notify_notification_new (_("Additional software was installed"), message, NULL);
+        notify_notification_set_app_name (notification, _("Software Updates"));
         notify_notification_set_timeout (notification, NOTIFY_EXPIRES_NEVER);
         notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
 
@@ -327,7 +330,7 @@ install_packages_cb (GObject *object,
         error_code = pk_results_get_error_code (results);
         if (error_code != NULL) {
                 g_warning ("failed to install file: %s, %s",
-                           pk_error_enum_to_text (pk_error_get_code (error_code)),
+                           pk_error_enum_to_string (pk_error_get_code (error_code)),
                            pk_error_get_details (error_code));
                 goto out;
         }
@@ -473,7 +476,7 @@ check_available (GsdUpdatesFirmware *firmware, const gchar *filename)
         error_code = pk_results_get_error_code (results);
         if (error_code != NULL) {
                 g_warning ("failed to search file: %s, %s",
-                           pk_error_enum_to_text (pk_error_get_code (error_code)),
+                           pk_error_enum_to_string (pk_error_get_code (error_code)),
                            pk_error_get_details (error_code));
                 goto out;
         }
@@ -578,6 +581,7 @@ delay_timeout_cb (gpointer data)
 
         /* TRANSLATORS: title of libnotify bubble */
         notification = notify_notification_new (_("Additional firmware required"), string->str, NULL);
+        notify_notification_set_app_name (notification, _("Software Updates"));
         notify_notification_set_timeout (notification, NOTIFY_EXPIRES_NEVER);
         notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
         notify_notification_add_action (notification, "install-firmware",
