@@ -152,7 +152,7 @@ get_distro_upgrades_finished_cb (GObject *object,
         error_code = pk_results_get_error_code (results);
         if (error_code != NULL) {
                 g_warning ("failed to get upgrades: %s, %s",
-                           pk_error_enum_to_text (pk_error_get_code (error_code)),
+                           pk_error_enum_to_string (pk_error_get_code (error_code)),
                            pk_error_get_details (error_code));
                 goto out;
         }
@@ -184,7 +184,7 @@ get_distro_upgrades_finished_cb (GObject *object,
                               NULL);
                 g_string_append_printf (string, "%s (%s)\n",
                                         name,
-                                        pk_distro_upgrade_enum_to_text (state));
+                                        pk_distro_upgrade_enum_to_string (state));
                 g_free (name);
         }
         if (string->len != 0)
@@ -193,10 +193,7 @@ get_distro_upgrades_finished_cb (GObject *object,
         /* TRANSLATORS: a distro update is available, e.g. Fedora 8 to Fedora 9 */
         title = _("Distribution upgrades available");
         notification = notify_notification_new (title, string->str, NULL);
-        if (notification == NULL) {
-                g_warning ("failed to create notification");
-                goto out;
-        }
+        notify_notification_set_app_name (notification, _("Software Updates"));
         notify_notification_set_timeout (notification, NOTIFY_EXPIRES_NEVER);
         notify_notification_set_urgency (notification, NOTIFY_URGENCY_NORMAL);
         notify_notification_add_action (notification, "distro-upgrade-info",
@@ -257,7 +254,7 @@ refresh_cache_finished_cb (GObject *object, GAsyncResult *res, GsdUpdatesManager
         error_code = pk_results_get_error_code (results);
         if (error_code != NULL) {
                 g_warning ("failed to refresh the cache: %s, %s",
-                           pk_error_enum_to_text (pk_error_get_code (error_code)),
+                           pk_error_enum_to_string (pk_error_get_code (error_code)),
                            pk_error_get_details (error_code));
                 goto out;
         }
@@ -319,10 +316,7 @@ notify_critical_updates (GsdUpdatesManager *manager, GPtrArray *array)
         /* do the bubble */
         g_debug ("title=%s, message=%s", title, message);
         notification = notify_notification_new (title, message, NULL);
-        if (notification == NULL) {
-                g_warning ("failed to get bubble");
-                return;
-        }
+        notify_notification_set_app_name (notification, _("Software Updates"));
         notify_notification_set_timeout (notification, 15000);
         notify_notification_set_urgency (notification, NOTIFY_URGENCY_CRITICAL);
         notify_notification_add_action (notification, "show-update-viewer",
@@ -379,10 +373,7 @@ notify_normal_updates_maybe (GsdUpdatesManager *manager, GPtrArray *array)
         /* do the bubble */
         g_debug ("title=%s, message=%s", title, message);
         notification = notify_notification_new (title, message, NULL);
-        if (notification == NULL) {
-                g_warning ("failed to get bubble");
-                return;
-        }
+        notify_notification_set_app_name (notification, _("Software Updates"));
         notify_notification_set_timeout (notification, 15000);
         notify_notification_set_urgency (notification, NOTIFY_URGENCY_NORMAL);
         notify_notification_add_action (notification, "show-update-viewer",
@@ -437,6 +428,7 @@ update_check_on_battery (GsdUpdatesManager *manager)
         /* TRANSLATORS: informs user will not install by default */
         notification = notify_notification_new (_("Updates not installed"),
                                                 message, NULL);
+        notify_notification_set_app_name (notification, _("Software Updates"));
         notify_notification_set_timeout (notification, 15000);
         notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
         notify_notification_add_action (notification, "update-all-packages",
@@ -518,8 +510,8 @@ notify_update_finished (GsdUpdatesManager *manager, PkResults *results)
                               NULL);
 
                 split = pk_package_id_split (package_id);
-                g_debug ("%s, %s, %s", pk_info_enum_to_text (info),
-                           split[PK_PACKAGE_ID_NAME], summary);
+                g_debug ("%s, %s, %s", pk_info_enum_to_string (info),
+                         split[PK_PACKAGE_ID_NAME], summary);
                 if (info == PK_INFO_ENUM_BLOCKED) {
                         skipped_number++;
                         g_string_append_printf (message_text, "<b>%s</b> - %s\n",
@@ -567,6 +559,7 @@ notify_update_finished (GsdUpdatesManager *manager, PkResults *results)
         /* TRANSLATORS: title: system update completed all okay */
         notification = notify_notification_new (_("The system update has completed"),
                                                  message_text->str, NULL);
+        notify_notification_set_app_name (notification, _("Software Updates"));
         notify_notification_set_timeout (notification, 15000);
         notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
         if (restart == PK_RESTART_ENUM_SYSTEM) {
@@ -610,7 +603,7 @@ update_packages_finished_cb (PkTask *task,
         error_code = pk_results_get_error_code (results);
         if (error_code != NULL) {
                 g_warning ("failed to update system: %s, %s",
-                           pk_error_enum_to_text (pk_error_get_code (error_code)),
+                           pk_error_enum_to_string (pk_error_get_code (error_code)),
                            pk_error_get_details (error_code));
                 goto out;
         }
@@ -653,10 +646,7 @@ notify_failed_get_updates_maybe (GsdUpdatesManager *manager)
         button = _("Try again");
 
         notification = notify_notification_new (title, message, NULL);
-        if (notification == NULL) {
-                g_warning ("failed to create notification");
-                goto out;
-        }
+        notify_notification_set_app_name (notification, _("Software Updates"));
         notify_notification_set_timeout (notification, 120*1000);
         notify_notification_set_urgency (notification, NOTIFY_URGENCY_NORMAL);
         notify_notification_add_action (notification, "show-update-viewer",
@@ -698,7 +688,7 @@ package_download_finished_cb (GObject *object,
         error_code = pk_results_get_error_code (results);
         if (error_code != NULL) {
                 g_warning ("failed to download: %s, %s",
-                           pk_error_enum_to_text (pk_error_get_code (error_code)),
+                           pk_error_enum_to_string (pk_error_get_code (error_code)),
                            pk_error_get_details (error_code));
                 notify_failed_get_updates_maybe (manager);
                 goto out;
@@ -771,7 +761,7 @@ get_updates_finished_cb (GObject *object,
         error_code = pk_results_get_error_code (results);
         if (error_code != NULL) {
                 g_warning ("failed to get updates: %s, %s",
-                           pk_error_enum_to_text (pk_error_get_code (error_code)),
+                           pk_error_enum_to_string (pk_error_get_code (error_code)),
                            pk_error_get_details (error_code));
                 notify_failed_get_updates_maybe (manager);
                 goto out;
@@ -1243,15 +1233,6 @@ out:
         g_object_unref (root);
 }
 
-static void
-handle_method_call (GDBusConnection *connection_, const gchar *sender,
-                    const gchar *object_path, const gchar *interface_name,
-                    const gchar *method_name, GVariant *parameters,
-                    GDBusMethodInvocation *invocation, gpointer user_data)
-{
-        return;
-}
-
 static GVariant *
 handle_get_property (GDBusConnection *connection_, const gchar *sender,
                      const gchar *object_path, const gchar *interface_name,
@@ -1268,39 +1249,6 @@ handle_get_property (GDBusConnection *connection_, const gchar *sender,
         return retval;
 }
 
-static void
-on_bus_acquired (GDBusConnection *connection_, const gchar *name, gpointer user_data)
-{
-        guint registration_id;
-        GsdUpdatesManager *manager = GSD_UPDATES_MANAGER(user_data);
-        static const GDBusInterfaceVTable interface_vtable = {
-                handle_method_call,
-                handle_get_property,
-                NULL
-        };
-
-        registration_id = g_dbus_connection_register_object (connection_,
-                                                             "/",
-                                                             manager->priv->introspection->interfaces[0],
-                                                             &interface_vtable,
-                                                             NULL,  /* user_data */
-                                                             NULL,  /* user_data_free_func */
-                                                             NULL); /* GError** */
-        g_assert (registration_id > 0);
-}
-
-static void
-on_name_acquired (GDBusConnection *connection, const gchar *name, gpointer user_data)
-{
-        g_debug ("acquired name: %s", name);
-}
-
-static void
-on_name_lost (GDBusConnection *connection, const gchar *name, gpointer user_data)
-{
-        g_debug ("lost name: %s", name);
-}
-
 static void
 emit_changed (GsdUpdatesManager *manager)
 {
@@ -1325,6 +1273,39 @@ emit_changed (GsdUpdatesManager *manager)
         }
 }
 
+static const GDBusInterfaceVTable interface_vtable =
+{
+        NULL, /* MethodCall */
+        handle_get_property, /* GetProperty */
+        NULL, /* SetProperty */
+};
+
+static void
+on_bus_gotten (GObject *source_object,
+               GAsyncResult *res,
+               GsdUpdatesManager *manager)
+{
+        GDBusConnection *connection;
+        GError *error = NULL;
+
+        connection = g_bus_get_finish (res, &error);
+        if (connection == NULL) {
+                g_warning ("Could not get session bus: %s",
+                           error->message);
+                g_error_free (error);
+                return;
+        }
+        manager->priv->connection = connection;
+
+        g_dbus_connection_register_object (connection,
+                                           "/",
+                                           manager->priv->introspection->interfaces[0],
+                                           &interface_vtable,
+                                           manager,
+                                           NULL,
+                                           NULL);
+}
+
 gboolean
 gsd_updates_manager_start (GsdUpdatesManager *manager,
                            GError **error)
@@ -1417,14 +1398,11 @@ gsd_updates_manager_start (GsdUpdatesManager *manager,
         if (manager->priv->introspection == NULL)
                 goto out;
 
-        /* own the object */
-        manager->priv->owner_id = g_bus_own_name (G_BUS_TYPE_SESSION,
-                                                  "org.gnome.SettingsDaemonUpdates",
-                                                  G_BUS_NAME_OWNER_FLAGS_NONE,
-                                                  on_bus_acquired,
-                                                  on_name_acquired,
-                                                  on_name_lost,
-                                                  manager, NULL);
+        /* export the object */
+        g_bus_get (G_BUS_TYPE_SESSION,
+                   NULL,
+                   (GAsyncReadyCallback) on_bus_gotten,
+                   manager);
 
         /* success */
         ret = TRUE;
