@@ -40,17 +40,20 @@
 #include <X11/extensions/XInput.h>
 
 #include "gsd-enums.h"
+#include "gsd-input-helper.h"
 #include "gnome-settings-profile.h"
 #include "gsd-wacom-manager.h"
 
 /* Maximum number of buttons map-able. */
 #define WACOM_MAX_BUTTONS 32
 /* Device types to apply a setting to */
-#define WACOM_TYPE_STYLUS       (1 << 1)
-#define WACOM_TYPE_ERASER       (1 << 2)
-#define WACOM_TYPE_CURSOR       (1 << 3)
-#define WACOM_TYPE_PAD          (1 << 4)
-#define WACOM_TYPE_ALL          WACOM_TYPE_STYLUS | WACOM_TYPE_ERASER | WACOM_TYPE_CURSOR | WACOM_TYPE_PAD
+typedef enum {
+        WACOM_TYPE_STYLUS =     (1 << 1),
+        WACOM_TYPE_ERASER =     (1 << 2),
+        WACOM_TYPE_CURSOR =     (1 << 3),
+        WACOM_TYPE_PAD    =     (1 << 4),
+        WACOM_TYPE_ALL    =     WACOM_TYPE_STYLUS | WACOM_TYPE_ERASER | WACOM_TYPE_CURSOR | WACOM_TYPE_PAD
+} WacomType;
 
 #define GSD_WACOM_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_WACOM_MANAGER, GsdWacomManagerPrivate))
 
@@ -75,19 +78,21 @@
 
 struct GsdWacomManagerPrivate
 {
+        guint start_idle_id;
         GSettings *wacom_settings;
         GSettings *stylus_settings;
         GSettings *eraser_settings;
         GSettings *cursor_settings;
         GSettings *pad_settings;
         GdkDeviceManager *device_manager;
+        guint device_added_id;
 };
 
 static void     gsd_wacom_manager_class_init  (GsdWacomManagerClass *klass);
 static void     gsd_wacom_manager_init        (GsdWacomManager      *wacom_manager);
 static void     gsd_wacom_manager_finalize    (GObject              *object);
 static void     set_wacom_settings            (GsdWacomManager      *manager);
-static XDevice* device_is_wacom               (const gint            type,
+static XDevice* device_is_wacom               (WacomType         type,
                                                const XDeviceInfo deviceinfo);
 
 G_DEFINE_TYPE (GsdWacomManager, gsd_wacom_manager, G_TYPE_OBJECT)
@@ -96,8 +101,8 @@ static gpointer manager_object = NULL;
 
 static GObject *
 gsd_wacom_manager_constructor (GType                     type,
-                              guint                      n_construct_properties,
-                              GObjectConstructParam     *construct_properties)
+                               guint                      n_construct_properties,
+                               GObjectConstructParam     *construct_properties)
 {
         GsdWacomManager      *wacom_manager;
 
@@ -143,13 +148,13 @@ set_devicepresence_handler (GsdWacomManager *manager)
         if (device_manager == NULL)
                 return;
 
-        g_signal_connect (G_OBJECT (device_manager), "device-added",
-                          G_CALLBACK (device_added_cb), manager);
+        manager->priv->device_added_id = g_signal_connect (G_OBJECT (device_manager), "device-added",
+                                                           G_CALLBACK (device_added_cb), manager);
         manager->priv->device_manager = device_manager;
 }
 
 static gboolean
-device_is_type (const gint type,
+device_is_type (WacomType          type,
                 const XDeviceInfo *dev)
 {
         static Atom stylus, cursor, eraser, pad;
@@ -179,7 +184,7 @@ device_is_type (const gint type,
 }
 
 static XDevice*
-device_is_wacom (const gint type,
+device_is_wacom (WacomType         type,
                  const XDeviceInfo deviceinfo)
 {
         XDevice *device;
@@ -223,71 +228,9 @@ device_is_wacom (const gint type,
         return device;
 }
 
-/* Generic property setting code. Fill up the struct property with the property
- * data and pass it into device_set_property together with the device to be
- * changed.  Note: doesn't cater for non-zero offsets yet, but we don't have
- * any settings that require that.
- */
-struct property {
-        const gchar *name;      /* property name */
-        gint nitems;            /* number of items in data */
-        gint format;            /* 8 or 32 */
-        union {
-                const gchar *c; /* 8 bit data */
-                const gint *i;  /* 32 bit data */
-        } data;
-};
-
 static void
-device_set_property (XDevice            *device,
-                     const char         *device_name,
-                     struct property    *property)
-{
-        int rc;
-        Atom prop;
-        Atom realtype;
-        int realformat;
-        unsigned long nitems, bytes_after;
-        unsigned char *data;
-
-        prop = XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
-                            property->name, False);
-        if (!prop)
-                return;
-
-        gdk_error_trap_push ();
-
-        rc = XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
-                                 device, prop, 0, property->nitems, False,
-                                 XA_INTEGER, &realtype, &realformat, &nitems,
-                                 &bytes_after, &data);
-
-        if (rc == Success && realtype == XA_INTEGER &&
-            realformat == property->format && nitems >= property->nitems) {
-                int i;
-                for (i = 0; i < nitems; i++) {
-                        switch (property->format) {
-                                case 8:
-                                        data[i] = property->data.c[i];
-                                        break;
-                                case 32:
-                                        ((long*)data)[i] = property->data.i[i];
-                                        break;
-                        }
-                }
-
-                XChangeDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
-                                       device, prop, XA_INTEGER, realformat,
-                                       PropModeReplace, data, nitems);
-        }
-
-        if (gdk_error_trap_pop ())
-                g_warning ("Error in setting \"%s\" for \"%s\"", property->name, device_name);
-}
-
-static void
-wacom_set_property (gint wacom_type,
-                    struct property *property)
+wacom_set_property (WacomType wacom_type,
+                    PropertyHelper *property)
 {
         XDeviceInfo *device_info;
         gint n_devices;
@@ -315,7 +258,7 @@ static void
 set_rotation (GsdWacomRotation rotation)
 {
         gchar rot = rotation;
-        struct property property = {
+        PropertyHelper property = {
                 .name = "Wacom Rotation",
                 .nitems = 1,
                 .format = 8,
@@ -326,10 +269,10 @@ set_rotation (GsdWacomRotation rotation)
 }
 
 static void
-set_pressurecurve (const gint    wacom_type,
+set_pressurecurve (WacomType      wacom_type,
                    GVariant      *value)
 {
-        struct property property = {
+        PropertyHelper property = {
                 .name = "Wacom Pressurecurve",
                 .nitems = 4,
                 .format = 32,
@@ -350,10 +293,10 @@ set_pressurecurve (const gint    wacom_type,
  * usable area of the physical device to the given area (in device coords)
  */
 static void
-set_area (const gint      wacom_type,
+set_area (WacomType        wacom_type,
           GVariant        *value)
 {
-        struct property property = {
+        PropertyHelper property = {
                 .name = "Wacom Tablet Area",
                 .nitems = 4,
                 .format = 32,
@@ -371,7 +314,8 @@ set_area (const gint      wacom_type,
 }
 
 static void
-set_absolute (const gint wacom_type, gint is_absolute)
+set_absolute (WacomType wacom_type,
+              gint      is_absolute)
 {
         XDeviceInfo *device_info;
         gint n_devices;
@@ -400,8 +344,8 @@ set_absolute (const gint wacom_type, gint is_absolute)
 }
 
 static void
-set_device_buttonmap (gint wacom_type,
-                      GVariant *value)
+set_device_buttonmap (WacomType  wacom_type,
+                      GVariant  *value)
 {
         XDeviceInfo *device_info;
         gint n_devices;
@@ -449,7 +393,7 @@ static void
 set_touch (gboolean touch)
 {
         gchar data = touch;
-        struct property property = {
+        PropertyHelper property = {
                 .name = "Wacom Enable Touch",
                 .nitems = 1,
                 .format = 8,
@@ -462,8 +406,12 @@ set_touch (gboolean touch)
 static void
 set_tpcbutton (gboolean tpcbutton)
 {
-        gchar data = tpcbutton;
-        struct property property = {
+        /* Wacom's TPCButton option which this setting emulates is to enable
+         * Tablet PC stylus behaviour when on. The property "Hover Click"
+         * works the other way round, i.e. if Hover Click is enabled this
+         * is the equivalent of TPC behaviour disabled. */
+        gchar data = tpcbutton ? 0 : 1;
+        PropertyHelper property = {
                 .name = "Wacom Hover Click",
                 .nitems = 1,
                 .format = 8,
@@ -474,9 +422,10 @@ set_tpcbutton (gboolean tpcbutton)
 }
 
 static void
-set_pressurethreshold (const gint wacom_type, gint threshold)
+set_pressurethreshold (WacomType wacom_type,
+                       gint      threshold)
 {
-        struct property property = {
+        PropertyHelper property = {
                 .name = "Wacom Pressure Threshold",
                 .nitems = 1,
                 .format = 32,
@@ -573,6 +522,8 @@ gsd_wacom_manager_idle_cb (GsdWacomManager *manager)
 
         gnome_settings_profile_end (NULL);
 
+        manager->priv->start_idle_id = 0;
+
         return FALSE;
 }
 
@@ -582,7 +533,7 @@ gsd_wacom_manager_start (GsdWacomManager *manager,
 {
         gnome_settings_profile_start (NULL);
 
-        g_idle_add ((GSourceFunc) gsd_wacom_manager_idle_cb, manager);
+        manager->priv->start_idle_id = g_idle_add ((GSourceFunc) gsd_wacom_manager_idle_cb, manager);
 
         gnome_settings_profile_end (NULL);
 
@@ -597,7 +548,7 @@ gsd_wacom_manager_stop (GsdWacomManager *manager)
         g_debug ("Stopping wacom manager");
 
         if (p->device_manager != NULL) {
-                g_object_unref (p->device_manager);
+                g_signal_handler_disconnect (p->device_manager, p->device_added_id);
                 p->device_manager = NULL;
         }
 
@@ -639,6 +590,9 @@ gsd_wacom_manager_finalize (GObject *object)
 
         g_return_if_fail (wacom_manager->priv != NULL);
 
+        if (wacom_manager->priv->start_idle_id != 0)
+                g_source_remove (wacom_manager->priv->start_idle_id);
+
         G_OBJECT_CLASS (gsd_wacom_manager_parent_class)->finalize (object);
 }
 
