@@ -87,6 +87,7 @@ struct _TranslationEntry {
 
 struct GnomeXSettingsManagerPrivate
 {
+        guint              start_idle_id;
         XSettingsManager **managers;
         GHashTable        *settings;
 
@@ -469,6 +470,8 @@ start_fontconfig_monitor_idle_cb (GnomeXSettingsManager *manager)
 
         gnome_settings_profile_end (NULL);
 
+        manager->priv->start_idle_id = 0;
+
         return FALSE;
 }
 
@@ -479,7 +482,7 @@ start_fontconfig_monitor (GnomeXSettingsManager  *manager)
 
         fontconfig_cache_init ();
 
-        g_idle_add ((GSourceFunc) start_fontconfig_monitor_idle_cb, manager);
+        manager->priv->start_idle_id = g_idle_add ((GSourceFunc) start_fontconfig_monitor_idle_cb, manager);
 
         gnome_settings_profile_end (NULL);
 }
@@ -580,12 +583,23 @@ setup_xsettings_managers (GnomeXSettingsManager *manager)
         int         n_screens;
         gboolean    res;
         gboolean    terminated;
+        gint        tries = 0;
 
         display = gdk_display_get_default ();
         n_screens = gdk_display_get_n_screens (display);
 
-        res = xsettings_manager_check_running (gdk_x11_display_get_xdisplay (display),
-                                               gdk_screen_get_number (gdk_screen_get_default ()));
+	/* FIXME: this is a temporary fix for https://bugzilla.gnome.org/show_bug.cgi?id=634988
+           Needs to be removed as soon as a proper fix lands in gdm
+	   (hopefully for 3.0.1 */
+        do {
+                res = xsettings_manager_check_running (gdk_x11_display_get_xdisplay (display),
+                                                       gdk_screen_get_number (gdk_screen_get_default ()));
+                if (res) {
+                        g_usleep (100000);
+                        tries++;
+                }
+        } while (res && tries <= 20);
+
         if (res) {
                 g_warning ("You can only run one xsettings manager at a time; exiting");
                 return FALSE;
@@ -804,6 +818,9 @@ gnome_xsettings_manager_finalize (GObject *object)
 
         g_return_if_fail (xsettings_manager->priv != NULL);
 
+        if (xsettings_manager->priv->start_idle_id != 0)
+                g_source_remove (xsettings_manager->priv->start_idle_id);
+
         G_OBJECT_CLASS (gnome_xsettings_manager_parent_class)->finalize (object);
 }
 
